unit MPFilterFormat;

interface
implementation
uses
  Windows,
  SysUtils,
  MPPlugin,
  MPPluginHeader,
  FastStrings;

function FastReplacePart(const ASource: String; const AStart, ALength: Integer;
                         const AReplace: String): String;
var
  iSrcLength: Integer;
  iLength:    Integer;
  iDiff:      Integer;
  iDest:      Integer;

begin
  iSrcLength  := Length(ASource);
  iLength     := Length(AReplace);
  iDiff       := iLength - ALength;
  iDest       := 1;

  SetLength(Result, iSrcLength + iDiff);

  // Write first part
  CopyMemory(@Result[iDest], @ASource[1], AStart - 1);
  Inc(iDest, AStart - 1);

  // Write replacement
  CopyMemory(@Result[iDest], @AReplace[1], iLength);
  Inc(iDest, iLength);

  // Write last part
  CopyMemory(@Result[iDest], @ASource[AStart + ALength],
             iSrcLength - AStart - (ALength - 1));
end;


procedure FilterFormat(var ASource: String; const AInfo: TMPFilterInfo);
  procedure ProcessColor(const ATag, AReplace: String);
  var
    iPos:       Integer;
    iEnd:       Integer;
    sTag:       String;
    iTagLen:    Integer;
    iColor:     Integer;
    sReplace:   String;

  begin
    sTag    := '(!' + ATag;
    iTagLen := Length(sTag);

    // Find tag start
    iPos    := FastPos(ASource, sTag, Length(ASource), iTagLen, 1);
    while iPos > 0 do begin
      // Find tag end
      iEnd  := FastPos(ASource, ')', Length(ASource), 1, iPos);

      if iEnd > 0 then begin
        // End found, check for valid color
        iColor  := StrToIntDef(Copy(ASource, iPos + iTagLen,
                                    iEnd - iPos - iTagLen), -1);

        if (iColor > 0) and (iColor < 100) then begin
          // Plus! allows two-digit numbers, anything above 15 just gets black
          if iColor > 15 then
            iColor  := 1;

          // Replace tag
          sReplace  := sCCColor + AReplace + IntToStr(iColor);
          ASource   := FastReplacePart(ASource, iPos, (iEnd - iPos) + 1, sReplace);
          Inc(iPos, Length(sReplace));
        end else
          // Ignore and continue
          Inc(iPos);
      end else
        // No end found, ignore and continue
        Inc(iPos);

      // Find tag start
      iPos  := FastPos(ASource, sTag, Length(ASource), iTagLen, iPos);
    end;
  end;

begin
  // Styles
  ASource := FastReplace(ASource, '(!FB)', sCCBold, True);
  ASource := FastReplace(ASource, '(!FI)', sCCItalic, True);
  ASource := FastReplace(ASource, '(!FU)', sCCUnderline, True);
  ASource := FastReplace(ASource, '(!FS)', sCCStrikeout, True);
  ASource := FastReplace(ASource, '(!FR)', sCCReset, True);

  // Colors
  ProcessColor('FC', '');
  ProcessColor('FG', ',');
end;


initialization
  RegisterFilter(FilterFormat);

end.
