unit MPFilterIP;

interface
implementation
uses
  SysUtils,
  Windows,
  Winsock,
  MPPlugin,
  FastStrings;

type
  TaPInAddr = array[0..250] of PInAddr;
  PaPInAddr = ^TaPInAddr;


procedure FilterIP(var ASource: String; const AInfo: TMPFilterInfo);
var
  wsData:     WSAData;
  cHost:      array[0..255] of Char;
  pHost:      PHostEnt;
  pAddresses: PaPInAddr;
  iAddress:   Integer;
  iIP:        TInAddr;
  sIP:        String;

begin
  sIP := '';
  
  // First a quick check to see if the IP should be retrieved (saves some
  // time when no IP is needed, FastPos is much faster than Winsock)
  if FastPos(ASource, '(!IP)', Length(ASource), 5, 1) = 0 then
    exit;

  if WSAStartup(MakeWord(1, 1), wsData) <> 0 then
    exit;

  try
    // Get local host
    FillChar(cHost, SizeOf(cHost), #0);
    if gethostname(@cHost, SizeOf(cHost)) <> 0 then
      exit;

    // Get local host info
    pHost := gethostbyname(@cHost);
    if not Assigned(pHost) then
      exit;

    // Parse address list
    pAddresses  := PaPInAddr(pHost^.h_addr_list);
    iAddress    := 0;
    iIP.S_addr  := 0;

    while pAddresses^[iAddress] <> nil do begin
      iIP.S_addr  := pAddresses^[iAddress]^.S_addr;
      Inc(iAddress);
    end;

    if iIP.S_addr > 0 then
      // Convert IP address
      with iIP.S_un_b do
        sIP := Format('%d.%d.%d.%d', [Integer(s_b1), Integer(s_b2),
                                      Integer(s_b3), Integer(s_b4)]);
  finally
    WSACleanup();
    ASource := FastReplace(ASource, '(!IP)', sIP, True);
  end;
end;


initialization
  RegisterFilter(FilterIP);

end.
