//////////////////////////////////////////////////////////////////////
//                                                                  //
// File: MPPluginHeader.h                                           //
// Content: Messenger Plus! plugin definitions                      //
// Author: Patchou                                                  //
//                                                                  //
// Last Update: 2004/05/06                                          //
//                                                                  //
// Delphi conversion by M. van Renswoude (PsychoMark)               //
// Last update: 2004/05/24                                          //
//                                                                  //
//////////////////////////////////////////////////////////////////////
unit MPPluginHeader;

interface
uses
  Windows;

type
  //////////////////////////////////////////////////////////////////////
  //                                                                  //
  // Purpose: Parameter structure used by some functions              //
  //                                                                  //
  // Members:                                                         //
  //   * iConversationWnd: Messenger Conversation Window COM          //
  //     interface. This pointer can be used to retrieve an           //
  //     IMessengerConversationWnd interface from the Messenger API.  //
  //     Important: in some occasions, this parameter may be null.    //
  //     Also, you MUST NOT call Release() on this interface          //
  //     (Messenger Plus! will do it for you when the function        //
  //     returns).                                                    //
  //   * sContactName. When used, this parameter indicates the        //
  //     friendly name of the contact who's action is the source of   //
  //     the call.                                                    //
  //                                                                  //
  //////////////////////////////////////////////////////////////////////
  PLUGIN_PARAM    = record
    iConversationWnd:         IDispatch;
    sContactName:             array[0..511] of Char;
  end;
  TPluginParam    = PLUGIN_PARAM;
  PPluginParam    = ^TPluginParam;

  PLUGIN_PARAMW   = record
    iConversationWnd:         IDispatch;
    sContactName:             array[0..511] of WideChar;
  end;
  TPluginParamW   = PLUGIN_PARAMW;
  PPluginParamW   = ^TPluginParamW;


  //////////////////////////////////////////////////////////////////////
  //                                                                  //
  // Recognized plugin functions                                      //
  //                                                                  //
  // Note: functions that end with a "W" are widechar (unicode)       //
  // versions of the correspondent ANSI function. These methods are  //
  // called in Windows NT4/2000/XP when defined. You are not forced   //
  // to define them; if they don't exist, Messenger Plus! will do the //
  // proper character conversions and use the ANSI functions.         //
  //////////////////////////////////////////////////////////////////////

  //////////////////////////////////////////////////////////////////////
  //                                                                  //
  // Purpose: Initialization function                                 //
  //                                                                  //
  // Definition:                                                      //
  //   This function is called when Messenger starts and each time a  //
  //   different user signs-in in Messenger.                          //
  //                                                                  //
  // Parameters:                                                      //
  //   * nVersion: plugin support version. This will be used in the   //
  //     future to identify different levels of support. Messenger    //
  //     Plus! 2.50 sets this value to 3.                             //
  //   * sUserEmail: email of the current user in Messenger. If       //
  //     Messenger is not signed-in, this parameter is the email of   //
  //     the last user who signed in. Messenger Plus! uses this value //
  //     to load its own settings.                                    //
  //   * iMessengerObj: Messenger COM interface. This pointer         //
  //     can be used to retrieve an IMessenger3 interface from the    //
  //     Messenger API. For more information, consult the public      //
  //     Messenger API documentation on Microsoft's web site.         //
  //     Important: in some occasions, this parameter may be null.    //
  //     Also, you MUST NOT call Release() on this interface          //
  //     (Messenger Plus! will do it for you during uninitialization).//
  //   * return value: if your initialization is successfull, return  //
  //     TRUE to notify Messenger Plus! that the plugin is ready to   //
  //     use.                                                         //
  //                                                                  //
  // IMPORTANT: if this method is called in your plugin, if it        //
  // returns TRUE but Messenger Plus! still doesn't call your other   //
  // functions, the reason is probably that it detected that your     //
  // plugin doesn't export its functions correctly. ALL your          //
  // functions MUST be declared with the __stdcall keyword (included  //
  // in the MPPLUGIN_RETURN_xxx defines).                             //
  //                                                                  //
  // Plugin Version Index:                                            //
  //     Messenger Plus! 2.20: 1                                      //
  //     Messenger Plus! 2.21: 2                                      //
  //     Messenger Plus! 2.50: 3                                      //
  //	   Messenger Plus! 3.00: 4										                  //
  //                                                                  //
  //////////////////////////////////////////////////////////////////////
  {
  function Initialize(nVersion: DWORD; sUserEmail: PChar;
                      iMessengerObj: IDispatch): BOOL; stdcall;
  }


  //////////////////////////////////////////////////////////////////////
  //                                                                  //
  // Purpose: Uninitialization function                               //
  //                                                                  //
  // Definition:                                                      //
  //   This function is called when Messenger exits or when a         //
  //   different user signs-in in Messenger (before Initialize() is   //
  //   called for the new user).                                      //
  //                                                                  //
  //////////////////////////////////////////////////////////////////////
  {
  procedure Uninitialize(); stdcall;
  }

  //////////////////////////////////////////////////////////////////////
  //																	                                //
  // Purpose: Configuration/Information function						          //
  //																	                                //
  // Definition:														                          //
  //	 This function is called when the user clicks on "Configure" in //
  //   the Plugins section of the Preferences of Messenger Plus! 3.00 //
  //   and above. Use it to display a configuration panel or an about //
  //   box.															                              //
  //																	                                //
  //////////////////////////////////////////////////////////////////////
  {
  procedure Configure(); stdcall;
  }

  //////////////////////////////////////////////////////////////////////
  //                                                                  //
  // Purpose: Inform the users about the features of the plugin       //
  //                                                                  //
  // Definition:                                                      //
  //   This function is called by Messenger Plus! to get some         //
  //   information about the plugin. You are not forced to implement  //
  //   this function but doing so will help informing your users      //
  //   about the features you provide (your plugin will be listed in  //
  //   the Plus! menu along with its commands and tags).              //
  //                                                                  //
  // Parameters:                                                      //
  //   * sPluginName: copy the name of your plugin in this buffer.    //
  //     Your plugin will be listed using this name in the Plugins    //
  //     menu. The maximum size for this parameter is 128 characters. //
  //   * aCommands: list of commands supported by your plugin.        //
  //     - nCount: number of commands you set in in the other values. //
  //     - sName: name you want to be displayed for the command in    //
  //       the Plugins menu.                                          //
  //     - sValue: command string. Do NOT include "/" at the          //
  //       beginning of the command. Example: "xping".                //
  //     - sHelp: optional help you want to be displayed when the     //
  //       user selects the command from the Plugins menu. This is    //
  //       useful only when the command accepts arguments.            //
  //   * aTags: list of tags supported by your plugin.                //
  //     - nCount: number of tags you set in in the other values.     //
  //     - sName: name you want to be displayed for the tag in        //
  //       the Plugins menu.                                          //
  //     - sValue: tag string. This parameter is the complete tag,    //
  //       including the parenthesis. Remember that it is case        //
  //       sensitive. Example: "(!XSAMP)".                            //
  //   * return value: if you want Messenger Plus! to display all the //
  //     information to the user in the Plugins menu, return TRUE,    //
  //     else, return FALSE.                                          //
  //                                                                  //
  //  Note that the commands and the tags will be listed in the order //
  //  you put them in the lists. Also, it is a good practice to add   //
  //  a command, with the name of your plugin, that displays a        //
  //  readme, an about box, or any other kind of help.                //
  //                                                                  //
  //////////////////////////////////////////////////////////////////////
  PLUGIN_PUBLISH_LIST   = record
    nCount:                   Integer;
    sName:                    array[0..49, 0..127] of Char;
    sValue:                   array[0..49, 0..127] of Char;
    sHelp:                    array[0..49, 0..255] of Char;
  end;
  TPluginPublishList    = PLUGIN_PUBLISH_LIST;
  PPluginPublishList    = ^TPluginPublishList;

  PLUGIN_PUBLISH_LISTW  = record
    nCount:                   Integer;
    sName:                    array[0..49, 0..127] of WideChar;
    sValue:                   array[0..49, 0..127] of WideChar;
    sHelp:                    array[0..49, 0..255] of WideChar;
  end;
  TPluginPublishListW   = PLUGIN_PUBLISH_LISTW;
  PPluginPublishListW   = ^TPluginPublishListW;

  {
  function PublishInfo(const sPluginName: PChar;
                       const aCommands, aTags: PPluginPublishList): BOOL; stdcall;

  function PublishInfoW(const sPluginName: PWideChar;
                        const aCommands, aTags: PPluginPublishListW): BOOL; stdcall;
  }


  //////////////////////////////////////////////////////////////////////
  //                                                                  //
  // Purpose: Provide new commands in Messenger Plus!                 //
  //                                                                  //
  // Definition:                                                      //
  //   This function is called everytime Messenger Plus! needs its    //
  //   plugins to analyze a command. This happens when the following  //
  //   two conditions are met:                                        //
  //      * the user entered a message and chose to send it           //
  //      * the message begins by "/x". Commands defined by plugins   //
  //        must begin with an "x" (and a command always begins by a  //
  //        "/").                                                     //
  //                                                                  //
  // Parameters:                                                      //
  //   * sCommand: the command entered by the user, including the "/" //
  //     character. Note that commands are not case sensitive.        //
  //   * sCommandArg: all the text that followed the space after the  //
  //     command.                                                     //
  //   * pParam: additional information sent by Messenger Plus!. For  //
  //     more information, read the documentation of the PLUGIN_PARAM //
  //     structure. iConversationWnd is used.                         //
  //   * sResult: if you recognize the command passed in sCommand,    //
  //     you must copy the new text to send in this parameter. If     //
  //     you copy an empty string, the text will be ignored and won't //
  //     be sent on the network. Important: the size of this buffer   //
  //     is 4096 characters and it is YOUR responsability to verify   //
  //     that you don't write more characters to avoid memory faults. //
  //   * return value: if you recognized the command and filled the   //
  //     sResult buffer, return TRUE, else, return FALSE.             //
  //                                                                  //
  // Example:                                                         //
  //   The user sends the following text: "/xquote hello you".        //
  //   sCommand = "/xquote"                                           //
  //   sCommandArg = "hello you"                                      //
  //   You could return in sResult this text: "I salute you"          //
  //                                                                  //
  // Advanced: if the plugin version returned by Initialize() is 2 or //
  // above, you can specify more than one message or command in       //
  // sResult. Simply create new lines with # as first character (use  //
  // '\n' to create new lines in your string). Up to 20 actions can   //
  // be specified, each being executed at a 0.5 second interval. A    //
  // different delay can even be entered for each action. After the   //
  // first #, just put a digit between 1 and 9 and another #.         //
  //   Example: sResult = "#First message\n#Second msg\n#5#Third msg" //
  //                                                                  //
  // Important: keep in mind that several plugins can be installed    //
  // and that the user sends a lot of text. If you don't support      //
  // the command passed in parameter, you must return as fast as      //
  // possible and without doing anything else.                        //
  //                                                                  //
  //////////////////////////////////////////////////////////////////////
  {
  function ParseCommand(const sCommand, sCommandArg: PChar;
                        pParam: PPluginParam; sResult: PChar): BOOL; stdcall;

  function ParseCommandW(const sCommand, sCommandArg: PWideChar;
                         pParam: PPluginParamW; sResult: PWideChar): BOOL; stdcall;
  }


  //////////////////////////////////////////////////////////////////////
  //                                                                  //
  // Purpose: Provide new tags in Messenger Plus!                     //
  //                                                                  //
  // Definition:                                                      //
  //   This function is called everytime Messenger Plus! needs its    //
  //   plugins to analyze a tag. This happens when the following two  //
  //   conditions are met:                                            //
  //      * the user entered a message and chose to send it           //
  //      * the message contains a tag that begins by "x" or "X". A   //
  //        tag is always formatted this way: "(!tag)", is case       //
  //        sensitive and doesn't contain space characters. So plugin //
  //        tags must look like "(!xtag)" (where "tag" can be         //
  //        anything you want).                                       //
  //                                                                  //
  // Parameters:                                                      //
  //   * sTag: the tag entered by the user, containing the            //
  //     parenthesis.                                                 //
  //   * pParam: additional information sent by Messenger Plus!. For  //
  //     more information, read the documentation of the PLUGIN_PARAM //
  //     structure. iConversationWnd is used.                         //
  //   * sResult: if you recognize the tag passed in sTag, you        //
  //     must copy the text that will replace the tag in this         //
  //     parameter. Important: the size of this buffer is 2048        //
  //     characters and it is YOUR responsability to verify that you  //
  //     don't write more characters to avoid memory faults. In any   //
  //     case, no more than 1100 characters will be sent on the       //
  //     network, even if the original text contained several tags    //
  //     using, for example, 500 characters each.                     //
  //   * return value: if you recognized the tag and filled the       //
  //     sResult buffer, return TRUE, else, return FALSE.             //
  //                                                                  //
  // Example:                                                         //
  //   The user sends the following text: "my horoscope: (!XH)".      //
  //   sTag = "(!XH)"                                                 //
  //   You could return in sResult this text: "you'll meet somone     //
  //      today" and text sent on the network will be "my horoscope:  //
  //      you'll meet someone today".                                 //
  //                                                                  //
  // Important: keep in mind that several plugins can be installed    //
  // and that the user sends a lot of text. If you don't support      //
  // the tag passed in parameter, you must return as fast as          //
  // possible and without doing anything else.                        //
  //                                                                  //
  //////////////////////////////////////////////////////////////////////
  {
  function ParseTag(const sTag: PChar; pParam: PPluginParam;
                    sResult: PChar): BOOL; stdcall;

  function ParseTagW(const sTag: PWideChar; pParam: PPluginParamW;
                     sResult: PWideChar): BOOL; stdcall;
  }


  //////////////////////////////////////////////////////////////////////
  //                                                                  //
  // Purpose: Allow special actions when a plugin text is received    //
  //                                                                  //
  // Definition:                                                      //
  //   This function is called everytime Messenger Plus! detects a    //
  //   plugin text ID in a message received from a user. This ID is   //
  //   generally placed at the beginning of a text by a plugin        //
  //   command to perform an action on the destination computers.     //
  //   This can be used, for example, to play a sound. See the        //
  //   control characters section for more information.               //
  //                                                                  //
  // Parameters:                                                      //
  //   * sNotifyCode: this 5 character string is the notify code that //
  //     was sent next to the plugin ID control character. Use it to  //
  //     identify what action to perform.                             //
  //   * sText: the rest of the received message. You can use it to   //
  //     get more information about the action you want to perform.   //
  //     However, you MUST NOT use this parameter to determine if an  //
  //     action is supported by your plugin. Use sNotifyCode instead. //
  //   * pParam: additional information sent by Messenger Plus!. For  //
  //     more information, read the documentation of the PLUGIN_PARAM //
  //     structure. iConversationWnd and sContactName are used.       //
  //   * sTextToSend: this parameter is optional. You can use it if   //
  //     you want to ask Messenger Plus! to send a new message after  //
  //     this one has been displayed. This WON'T modify the message   //
  //     associated with the notify code, this will simply send a new //
  //     one. Important: the size of this buffer is 4096 characters   //
  //     and it is YOUR responsability to verify that you don't write //
  //     more characters to avoid memory faults.
  //   * return value: if you recognized the notify code return TRUE, //
  //     else, return FALSE.                                          //
  //                                                                  //
  // Advanced: if the plugin version returned by Initialize() is 2 or //
  // above, you can specify more than one message or command in       //
  // sTextToSend. Simply create new lines with # as first character   //
  // (use '\n' to create new lines in your string). Up to 20 actions  //
  // can be specified, each being executed at a 0.5 second interval.  //
  // A different delay can even be entered for each action. After the //
  // first #, just put a digit between 1 and 9 and another #.         //
  //   Example: sTextToSend = "#First message\n#5#Second message"     //
  //                                                                  //
  // Important: keep in mind that several plugins can be installed    //
  // and that a lot of text is received. If you don't support the     //
  // notify code passed in parameter, you must return as fast as      //
  // possible and without doing anything else.                        //
  //                                                                  //
  //////////////////////////////////////////////////////////////////////
  {
  function ReceiveNotify(const sNotifyCode, sText: PChar;
                         const pParam: PPluginParam;
                         sTextToSend: PChar): BOOL; stdcall;

  function ReceiveNotifyW(const sNotifyCode, sText: PWideChar;
                          const pParam: PPluginParamW;
                          sTextToSend: PWideChar): BOOL; stdcall;
  }


//////////////////////////////////////////////////////////////////////
//                                                                  //
// Messenger Plus! control characters                               //
//                                                                  //
//////////////////////////////////////////////////////////////////////
const
  //////////////////////////////////////////////////////////////////////
  //                                                                  //
  // The Notify control character is for exclusive use by plugins. It //
  // is generally inserted by plugin commands. This code must be at   //
  // the beginning of a message and must be followed by a 5           //
  // characters notify code unique to your plugin.                    //
  //                                                                  //
  // Example: "XsamplHello!" (where "X" is the 0x12 character)        //
  // When received, the user will only see "Hello!" and               //
  // ReceiveNotify() will be called with "sampl" in sNotifyCode.      //
  //                                                                  //
  //////////////////////////////////////////////////////////////////////
  sCCNotify       = #18;


  //////////////////////////////////////////////////////////////////////
  //                                                                  //
  // These control characters are used by Messenger Plus! to perform  //
  // different kind of actions. Feel free to use them in your own     //
  // messages.                                                        //
  //                                                                  //
  //////////////////////////////////////////////////////////////////////
  sCCColor        = #03;  // Ctrl+K multi-format
  sCCBold         = #02;  // Ctrl+B multi-format
  sCCItalic       = #05;  // Ctrl+T multi-format
  sCCUnderline    = #32;  // Ctrl+U multi-format
  sCCStrikeout    = #06;  // Ctrl+S multi-format
  sCCReset        = #15;  // Ctrl+O multi-format

  // When placed at the beginning of a message, this control character
  // asks Messenger Plus! to temporary disable emoticons for the
  // message.
  sCCNoIcon       = #08;

  // To display action message (/me), put these control characters at
  // the beginning and at the end of your message.
  sCCMeStart      = 'I ';
  sCCMeEnd        = '';



  //////////////////////////////////////////////////////////////////////
  //                                                                  //
  // Messenger Plus! utility functions                                //
  //                                                                  //
  //////////////////////////////////////////////////////////////////////

  //////////////////////////////////////////////////////////////////////
  //                                                                  //
  // If the plugin engine version is 3 or above (see Initialize()),   //
  // your plugin can call this function to display a toast popup. The //
  // function returns immediately, it doesn't wait for the toast to   //
  // be displayed. You should use the DisplayToast/DisplayToastW      //
  // functions and not use the MPL_TOAST_INFO structure yourself.     //
  //                                                                  //
  // Parameters:                                                      //
  //   * sMessage: the text that will be displayed in the middle of   //
  //     the toast. If the text is too long to fit the window, it     //
  //     will be truncated.                                           //
  //   * sTitle: the title displayed on top of the toast. If NULL,    //
  //     the default title is displayed.                              //
  //   * sProgram: a program, or web link, that will be executed if   //
  //     the user clicks on the toast. If NULL, clicking on the toast //
  //     will do nothing.                                             //
  //   * bPlaySound: set to true if you want Messenger Plus! to play  //
  //     the default sound associated for notifications in the        //
  //     preferences of the user.                                     //
  //                                                                  //
  //   Note: as the user may have configured Messenger Plus! not to   //
  //   display notifications when he/she is busy, do not assume that  //
  //   the toast will always be displayed.                            //
  //                                                                  //
  //   Technical Note: the message must be sent from the same thread  //
  //   Initialize() was called, else, Messenger Plus! will reject the //
  //   call for security reasons.                                     //
  //                                                                  //
  //////////////////////////////////////////////////////////////////////
  procedure DisplayToast(const sMessage, sTitle, sProgram: PChar;
                         const bPlaySound: Boolean);

  procedure DisplayToastW(const sMessage, sTitle, sProgram: PWideChar;
                          const bPlaySound: Boolean);


implementation
type
  MPL_TOAST_INFO  = record
    sTitle:                   PChar;
    sMessage:                 PChar;
    sProgram:                 PChar;
    sSoundFile:               Pointer;

    nReserved:                DWORD;
  end;
  TMPLToastInfo   = MPL_TOAST_INFO;
  PMPLToastInfo   = ^TMPLToastInfo;

  MPL_TOAST_INFOW   = record
    sTitle:                   PWideChar;
    sMessage:                 PWideChar;
    sProgram:                 PWideChar;
    sSoundFile:               Pointer;

    nReserved:                DWORD;
  end;
  TMPLToastInfoW  = MPL_TOAST_INFOW;
  PMPLToastInfoW  = ^TMPLToastInfoW;

var
  MP_DISPLAYTOAST:      Cardinal;


procedure DisplayToast;
var
  pInfo:      TMPLToastInfo;

begin
  pInfo.sMessage  := sMessage;
  pInfo.sTitle    := sTitle;
  pInfo.sProgram  := sProgram;

  if bPlaySound then
    pInfo.sSoundFile  := Pointer(-1)
  else
    pInfo.sSoundFile  := nil;

  SendMessage(HWND_BROADCAST, MP_DISPLAYTOAST, Integer(@pInfo),
              Integer(BOOL(False)));
end;

procedure DisplayToastW;
var
  pInfo:      TMPLToastInfoW;

begin
  pInfo.sMessage  := sMessage;
  pInfo.sTitle    := sTitle;
  pInfo.sProgram  := sProgram;

  if bPlaySound then
    pInfo.sSoundFile  := Pointer(-1)
  else
    pInfo.sSoundFile  := nil;

  SendMessage(HWND_BROADCAST, MP_DISPLAYTOAST, Integer(@pInfo),
              Integer(BOOL(True)));
end;


initialization
  MP_DISPLAYTOAST := RegisterWindowMessage('MessengerPlus_DisplayToast');

end.
