library PluginDemo;

uses
  Windows,
  Graphics,
  MPPluginHeader in 'Common\MPPluginHeader.pas',
  MPPlugin in 'Common\MPPlugin.pas',
  MessengerAPI_TLB in 'Common\MessengerAPI_TLB.pas' {,

  These have all become obsolete now that Plus! 3 has support for parsing
  tags in the output!

  MPFilterContacts in 'Common\MPFilterContacts.pas',
  MPFilterDateTime in 'Common\MPFilterDateTime.pas',
  MPFilterFormat in 'Common\MPFilterFormat.pas',
  MPFilterIP in 'Common\MPFilterIP.pas'},
  FConfig in 'Forms\FConfig.pas' {frmConfig};

type
  TDemoPlugin = class(TMPPlugin)
  private
    function CommandDemo(const AInfo: TMPCommandInfo): String;
    function CommandNotify(const AInfo: TMPCommandInfo): String;
    function CommandTags(const AInfo: TMPCommandInfo): String;
    function CommandConfig(const AInfo: TMPCommandInfo): String;

    function TagDemo(const AInfo: TMPCommandInfo): String;

    function NotifyTest(const AInfo: TMPCommandInfo): String;
  public
    procedure Initialize(); override;
    procedure Configure(); override;
  end;


{============================ TDemoPlugin
  Initialization
========================================}
procedure TDemoPlugin.Initialize;
begin
  // Don't call inherited btw, since Initialize is an abstract function
  PluginName  := 'Demo Plus! Plugin';

  // Register our command. The callback functions can have any name, but
  // for the sake of readability I'm using sensible names...
  Commands.Add('xdemo', '/xdemo - Run a demo command', CommandDemo, 'parameter here');
  Commands.Add('xnotify', '/xnotify - Run a notification command', CommandNotify);
  Commands.Add('xtags', '/xtags - Demonstrate Plus! tags in plugins', CommandTags);

  Tags.Add('xdemo', '(!xdemo) - Run a demo tag', TagDemo);

  // If the Plus! version is below 3.0, Configure is not supported yet so we
  // must register a command to do the same. For consistency it is actually
  // best to keep the command so people who upgrade later can still use it,
  // but for the sake of showing there is a Version property we'll demonstrate
  // it this way...
  if Version < mp300 then
    Commands.Add('xconfig', '/xconfig - Configure the plugin', CommandConfig); 

  Notify.Add('NTEST', NotifyTest);
end;

procedure TDemoPlugin.Configure;
begin
  // Show configuration form
  with TfrmConfig.Create(nil) do
    try
      ShowModal();
    finally
      Free();
    end;
end;


{============================ TDemoPlugin
  Commands & Tags
========================================}
function TDemoPlugin.CommandDemo;
begin
  Result  := MPColor('This is the demo plugin speaking!', RGB(128,0,128)) + #13#10 +
             MPBold('Command: ') + AInfo.Command + #13#10 +
             MPBold('Params: ') + AInfo.Params;
end;

function TDemoPlugin.CommandNotify;
begin
  Result  := MPNotify('NTEST', MPColor('This is the demo plugin notifying you!',
                                       clRed));
end;

function TDemoPlugin.CommandTags;
begin
  Result  := MPColor('Filters demo showing Plus! tags:', clRed) + #13#10 +
             MPBold('My name: ') + '(!NP)'#13#10 +
             MPBold('My email: ') + '(!MP)'#13#10 +
             MPBold('Your name(s): ') + '(!N)'#13#10 +
             MPBold('Your email(s): ') + '(!M)'#13#10 +
             MPBold('Current date: ') + '(!D)'#13#10 +
             MPBold('Current time: ') + '(!T)'#13#10 +
             MPBold('Format tags: ') + '(!FC3)gr(!FB)ee(!FB)n(!FR)'#13#10 +
             MPBold('IP: ') + '(!IP)';
end;

function TDemoPlugin.CommandConfig;
begin
  Configure();
end;


function TDemoPlugin.TagDemo;
begin
  Result  := MPColor('This is the demo plugin speaking!', clRed) + #13#10 +
             MPBold('Tag: ') + AInfo.Command + #13#10 +
             MPBold('Params: ') + AInfo.Params;
end;


function TDemoPlugin.NotifyTest;
begin
  Result  := MPColor('Demo plugin received the test notification!',
                      clRed) + #13#10 +
             MPBold('Triggered by: ') + AInfo.ContactName;
end;



begin
  // Finally, the one line that brings it all together. By registering our
  // class it will be initialized and destroyed automatically...
  RegisterPlugin(TDemoPlugin);
end.
 